@@ -2,11 +2,120 @@
 Provides the high-level classes and definition for an Assistant that can be implemented in Kodexa and run on an
 instance of the Kodexa platform
 """
-import uuid
 from typing import List, Optional
 
-from kodexa.model import ContentObject, DocumentStore, Document
-from kodexa.model.objects import BaseEvent, Store, Taxonomy
+from kodexa.model import ContentObject, Document
+from kodexa.model.objects import Store, Taxonomy, BaseEvent
+from kodexa.platform.client import DocumentStoreEndpoint
+import logging
+from kodexa.pipeline import Pipeline
+
+logger = logging.getLogger()
+
+
+def replace_option(option_type, option_value, event_helper, cache, kodexa_client):
+    logger.info("Replacing option")
+    if option_type == 'document':
+        # We need to download the document from the store and replace the option with it
+
+        full_url = option_value
+        logger.info(f"Breaking down the full url for the document {full_url}")
+
+        if '/' in full_url:
+            url_parts = full_url.split(':')
+
+            # Each part, split by "/"
+            version = url_parts[1].split("/")[0]
+            store_ref = f"{url_parts[0]}:{version}"
+            document_family_id = url_parts[1].split("/")[1]
+            logger.info(f"Downloading document parameter {store_ref}/{document_family_id}")
+
+            if store_ref != "" and document_family_id != "undefined":
+                try:
+
+                    logger.info(f"Using store {store_ref}")
+                    cache_key = f'store-{store_ref}'
+
+                    if cache_key not in cache:
+                        logger.info(f"Using store {store_ref}")
+                        from kodexa.platform.client import DocumentStoreEndpoint
+                        store: DocumentStoreEndpoint = kodexa_client.get_object_by_ref('store', store_ref)
+                        from kodexa.platform.client import DocumentFamilyEndpoint
+                        document_family: DocumentFamilyEndpoint = store.get_family(document_family_id)
+                        document = document_family.get_document()
+                        document.ref = full_url
+                        cache[cache_key] = document
+                        logger.info(f"Downloaded document")
+
+                    # We keep the ref of the document so we can use it later
+                    return cache[cache_key]
+
+                except Exception as e:
+                    logger.warning(f"Unable to access the document ({e})")
+                    return None
+            else:
+                logger.info(f"Document reference not available")
+                return None
+
+        elif len(full_url) > 0:
+            # We have an ID to a Content Object
+            return Document.from_kddb(event_helper.get_content_object(content_object_id=full_url).read())
+
+    if 'store' in option_type.lower():
+        logger.info(f"Getting Store object instance {option_type} as {option_value}")
+
+        # We need to create a remote store
+        store = kodexa_client.get_object_by_ref('store', option_value)
+        return store
+
+    if 'pipeline' in option_type.lower():
+        logger.info(f"Getting pipeline instance {option_type} as {option_value}")
+
+        # To create a pipeline to represent the option
+        pipeline = Pipeline()
+        pipeline.steps = []
+
+        if isinstance(option_value['steps'], list):
+
+            for index, step in enumerate(option_value['steps']):
+                logger.info(f"Creating step {step}")
+                pipeline.add_step(step['ref'], name=f'Step {index}', step_type=step['stepType'],
+                                  options=step['options'] if 'options' in step else {})
+
+        return pipeline
+
+    if option_type == 'taxonomy':
+        logger.info(f"Getting Taxonomy object instance {option_type} as {option_value}")
+
+        try:
+            taxonomy = kodexa_client.get_object_by_ref('taxo', option_value)
+        except Exception as e:
+            logger.warning(f"Unable to access the taxonomy ({e})")
+            return None
+        return taxonomy
+
+    logger.info("Returning option value")
+    return option_value
+
+
+def replace_options(step, event_helper, cache, kodexa_client):
+    logger.info(f"Determine is we have any options that need replacement")
+    # We need to document any documents that are part of the options
+    for key in step.option_types.keys():
+        logger.info(f"Check option type for key {key}")
+
+        if key in step.options:
+            logger.info(f"Type is {step.option_types[key]}")
+            if step.option_types[key].startswith('list:'):
+                result = []
+                logger.info("Is a list type")
+                for value in step.options[key]:
+                    result.append(
+                        replace_option(step.option_types[key].split(':')[1], value, event_helper, cache, kodexa_client))
+                step.options[key] = result
+            else:
+                step.options[key] = replace_option(step.option_types[key], step.options[key], event_helper,
+                                                   cache, kodexa_client)
 
 
 class AssistantMetadata:
@@ -14,11 +123,13 @@ class AssistantMetadata:
     A set of metadata for the assistant that can be made available on from the context
     """
 
-    def __init__(self, assistant_id: str, assistant_name: str):
+    def __init__(self, assistant_id: str, assistant_name: str, project_id: Optional[str] = None):
         self.assistant_id = assistant_id
         """The ID of the assistant"""
         self.assistant_name = assistant_name
         """The name of the assistant"""
+        self.project_id = project_id
+        """The ID of the project that the assistant is owned by"""
 
 
 class AssistantPipeline:
@@ -28,7 +139,7 @@ class AssistantPipeline:
 
     def __init__(self, pipeline, description=None, write_back_to_store: bool = False,
                  data_store: Optional[Store] = None,
-                 taxonomies: Optional[List[Taxonomy]] = None):
+                 taxonomies: Optional[List[Taxonomy]] = None, labels_to_apply: Optional[List[str]] = None):
         self.pipeline = pipeline
         """The pipeline to execute"""
         self.description = description
@@ -39,6 +150,8 @@ class AssistantPipeline:
         """Optionally the datastore that we want to extract the labelled content to"""
         self.taxonomies = taxonomies
         """Optionally a list of the taxonomies to use when extracting the labels"""
+        self.labels_to_apply = labels_to_apply
+        """Optionally a list of the labels to apply to the document when complete"""
 
 
 class AssistantIntent:
@@ -59,8 +172,8 @@ class AssistantResponse:
     event.
     """
 
-    def __init__(self, pipelines: List[AssistantPipeline] = None, text: Optional[str] = None, available_intents=None,
-                 output_document: Document = None):
+    def __init__(self, pipelines: List[AssistantPipeline] = None, text: Optional[str] = None,
+                 output_document: Document = None, enabled_logging=False):
         """
         Initialize the response from the assistant
 
@@ -68,11 +181,9 @@ class AssistantResponse:
             pipelines: zero or more pipelines that you want executed on the content object for which the
                        event was raised
             text: the to be presented with the response
-            available_intents: a list of the available intents that you can return for this document
+            enabled_logging: should logging be enabled for this response
             output_document: the output document, if the assistant has created a document directly
         """
-        if available_intents is None:
-            available_intents = []
         if pipelines is None:
             pipelines = []
 
@@ -82,19 +193,19 @@ class AssistantResponse:
         self.text = text
         """The text that will be provided back to the user from the assistant"""
 
-        self.available_intents = available_intents
-        """Any available intentions that the assistant will further respond to"""
-
         self.output_document = output_document
         """The output document, if the assistant has directly created one"""
 
+        self.enabled_logging = enabled_logging
+        """Should logging be enabled for this response"""
+
 
 class AssistantContext:
     """The Assistant Context provides a way to interact with additional services and capabilities
     while processing an event
     """
 
-    from kodexa.model import ContentEvent, DocumentStore
+    from kodexa.model import ContentEvent
 
     def __init__(self, metadata: AssistantMetadata, path_to_kodexa_metadata: str = 'kodexa.yml',
                  stores=None, content_provider=None, extension_pack_util=None):
@@ -156,7 +267,7 @@ class AssistantContext:
             options = {}
         return self.extension_pack_util.get_step(step, options)
 
-    def get_store(self, event: ContentEvent) -> DocumentStore:
+    def get_store(self, event: ContentEvent) -> DocumentStoreEndpoint:
         """
         Get a document store for the event (based on the document family ID)
 
@@ -171,8 +282,8 @@ class AssistantContext:
                 return store
 
         if event.document_family.store_ref is not None:
-            from kodexa import KodexaPlatform
-            return KodexaPlatform.get_object_instance(event.document_family.store_ref, 'store')
+            from kodexa import KodexaClient
+            return KodexaClient().get_object_by_ref('store', event.document_family.store_ref)
 
         raise Exception(f"Unable to get store ref {event.document_family.store_ref}")
 
@@ -200,5 +311,3 @@ class Assistant:
           AssistantResponse: the response to the event
 
         """
-
-        pass
