@@ -14,7 +14,8 @@ import msgpack
 from addict import Dict
 
 from kodexa.model.base import KodexaBaseModel
-from kodexa.model.objects import ContentObject
+from kodexa.model.objects import ContentObject, FeatureSet
+import deepdiff
 
 
 class Ref:
@@ -49,8 +50,9 @@ class ContentException(Dict):
     """A content exception represents an issue identified during labeling or validation at the document level"""
 
     def __init__(self, exception_type: str, message: str, severity: str = 'ERROR', tag: Optional[str] = None,
-                 group_uuid: Optional[str] = None, tag_uuid: Optional[str] = None,
-                 exception_details: Optional[str] = None, node_uuid: Optional[str] = None, *args, **kwargs):
+                 group_uuid: Optional[str] = None, tag_uuid: Optional[str] = None, exception_type_id: Optional[str] = None,
+                 exception_details: Optional[str] = None, node_uuid: Optional[str] = None, value: Optional[str] = None,
+                 *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.tag = tag
         self.message = message
@@ -60,6 +62,8 @@ class ContentException(Dict):
         self.exception_type = exception_type
         self.node_uuid = node_uuid
         self.severity = severity
+        self.value = value
+        self.exception_type_id = exception_type_id
 
 
 class Tag(Dict):
@@ -69,7 +73,7 @@ class Tag(Dict):
                  uuid: Optional[str] = None, data: Any = None, *args, confidence: Optional[float] = None,
                  group_uuid: Optional[str] = None, parent_group_uuid: Optional[str] = None,
                  cell_index: Optional[int] = None, index: Optional[int] = None, bbox: Optional[List[int]] = None,
-                 note: Optional[str] = None, status: Optional[str] = None, **kwargs):
+                 note: Optional[str] = None, status: Optional[str] = None, owner_uri: Optional[str] = None, **kwargs):
         super().__init__(*args, **kwargs)
         self.start: Optional[int] = start
         """The start position (zero indexed) of the content within the node, if None then label is applied to the whole node"""
@@ -79,7 +83,7 @@ class Tag(Dict):
         """A string representing the value that was labelled in the node"""
         self.data: Optional[Any] = data
         """Any data object (JSON serializable) that you wish to associate with the label"""
-        self.uuid: Optional[str] = uuid
+        self.uuid: Optional[str] = uuid or str(uuid.uuid4())
         """The UUID for this tag instance, this allows tags that are on different content nodes to be related through the same UUID"""
         self.confidence: Optional[float] = confidence
         """The confidence of the tag in a range of 0-1"""
@@ -97,7 +101,8 @@ class Tag(Dict):
         """A note that can be associated with the tag"""
         self.status: Optional[str] = status
         """The status of the tag, this can be passed to an attribute status during extraction"""
-
+        self.owner_uri: Optional[str] = owner_uri
+        """The URI of the owner (ie. model://kodexa/narrative:1.0.0 or user://pdodds)"""
         # Pull the cell index from the data to the tag if we have it in the data
         if self.cell_index is None:
             if data and 'cell_index' in data:
@@ -984,7 +989,7 @@ class ContentNode(object):
             use_all_content=False, node_only=None,
             fixed_position=None, data=None, separator=" ", tag_uuid: str = None, confidence=None, value=None,
             use_match=True, index=None, cell_index=None, group_uuid=None, parent_group_uuid=None, note=None,
-            status=None):
+            status=None, owner_uri=None):
         """
         This will tag (see Feature Tagging) the expression groups identified by the regular expression.
 
@@ -1018,6 +1023,7 @@ class ContentNode(object):
           parent_group_uuid: The parent group uuid for the tag
           note: a text note for the tag
           status: a status for the tag, this can be transistioned to an attribute status during extraction
+          owner_uri: the uri of the entity that created the tag (model vs user; example: model://cdad-healthcare/cdad-excel-model:1.0.0 or user://pdodds)
 
         >>> document.content_node.tag('is_cheese')
         """
@@ -1055,7 +1061,7 @@ class ContentNode(object):
                                                           data=node_data, uuid=tag_uuid, confidence=confidence,
                                                           index=index, parent_group_uuid=parent_group_uuid,
                                                           group_uuid=group_uuid, cell_index=cell_index, note=note,
-                                                          status=status))
+                                                          status=status, owner_uri=owner_uri))
                             return -1
                         if start < part_length <= end:
                             node_to_check.add_feature('tag', tag_to_apply,
@@ -1065,7 +1071,7 @@ class ContentNode(object):
                                                           data=node_data, uuid=tag_uuid, confidence=confidence,
                                                           index=index, parent_group_uuid=parent_group_uuid,
                                                           group_uuid=group_uuid, cell_index=cell_index, note=note,
-                                                          status=status))
+                                                          status=status, owner_uri=owner_uri))
 
                         end = end - part_length
                         content_length = content_length + part_length
@@ -1136,7 +1142,7 @@ class ContentNode(object):
                     node.add_feature('tag', tag_to_apply,
                                      Tag(data=data, uuid=get_tag_uuid(tag_uuid), confidence=confidence, value=value,
                                          index=index, parent_group_uuid=parent_group_uuid, group_uuid=group_uuid,
-                                         cell_index=cell_index, note=note, status=status))
+                                         cell_index=cell_index, note=note, status=status, owner_uri=owner_uri))
                 else:
                     if not use_all_content:
                         if node.content:
@@ -1158,7 +1164,7 @@ class ContentNode(object):
                                                      Tag(data=data, uuid=get_tag_uuid(tag_uuid), confidence=confidence,
                                                          value=value, index=index, parent_group_uuid=parent_group_uuid,
                                                          group_uuid=group_uuid, cell_index=cell_index, note=note,
-                                                         status=status))
+                                                         status=status, owner_uri=owner_uri))
                             else:
                                 if matches:
                                     for match in matches:
@@ -1536,12 +1542,14 @@ class ContentNode(object):
             if not node:
                 if (traverse == traverse.ALL or traverse == traverse.PARENT) and self.get_parent().get_parent():
                     try:
-                        potential_next_node = self.get_parent().get_parent().get_children()[self.get_parent().index + 1].get_children()[0]
+                        potential_next_node = \
+                            self.get_parent().get_parent().get_children()[self.get_parent().index + 1].get_children()[0]
                         if potential_next_node:
                             return potential_next_node
                     except:
                         # traverse additional layer
-                        potential_next_node = self.get_parent().get_parent().get_parent().get_children()[self.get_parent().get_parent().index + 1].get_children()[0].get_children()[0]
+                        potential_next_node = self.get_parent().get_parent().get_parent().get_children()[
+                            self.get_parent().get_parent().index + 1].get_children()[0].get_children()[0]
                         if potential_next_node:
                             return potential_next_node
                 return node
@@ -1707,11 +1715,137 @@ class ContentClassification(object):
                                      selector=dict_val.get('selector'), confidence=dict_val.get('confidence'))
 
 
+class FeatureSetDiff:
+    """
+    A utility class that can be used to diff two feature sets
+    """
+
+    def __init__(self, first_feature_set: FeatureSet, second_feature_set: FeatureSet):
+        self.first_feature_map = self.parse_feature_set(first_feature_set)
+        self.second_feature_map = self.parse_feature_set(second_feature_set)
+        self._differences = deepdiff.DeepDiff(self.first_feature_map, self.second_feature_map,
+                                              exclude_obj_callback=self.exclude_callback).to_dict()
+        self._changed_nodes = self.get_changed_nodes()
+
+    def get_differences(self):
+        """
+        :return: Data dictionaries that contains the differences of two feature sets
+        """
+        if 'type_changes' in self._differences:
+            self._differences.pop('type_changes')
+
+        return self._differences
+
+    def get_changed_nodes(self):
+        """
+        :return: Data dictionary of added and removed nodes
+        """
+        return self._changed_nodes
+
+    def get_exclude_paths(self):
+        """
+        :return: List of paths to exclude
+        """
+        return ['shape', 'group_uuid', 'uuid', 'parent_group_uuid', 'single']
+
+    def exclude_callback(self, path, key):
+        """
+        Checks if the key is to be exluceded from the diff
+        :param path: contains the values of that key
+        :param key: The key of the data dictionary to compare
+        :return: boolean
+        """
+        if any(re.search(exclude_key, key) for exclude_key in self.get_exclude_paths()):
+            return True
+        else:
+            return False
+
+    def parse_feature_set(self, feature_set: FeatureSet):
+        """
+        :param feature_set: The feature set to be parsed
+        :return: Dictionary of feature with the key as the nodeUuid
+        """
+        return {feature.get('nodeUuid'): feature for feature in feature_set.node_features}
+
+    def parsed_values_changed(self):
+        for key, value in self._differences.get('values_changed').items():
+            # Check if the old_value is stil in the second_feature_map. If it is remove the key
+            if key in self.second_feature_map.node_features:
+                self._differences.get('values_changed').remove(key)
+
+    def is_equal(self) -> bool:
+        """
+        Checks if the two feature set is equal to each other
+        :return: This returns a bool
+        """
+        return self._differences == {}
+
+    def get_changed_nodes(self):
+        """
+        :return: A list of nodes that were changed
+        """
+        if self.is_equal():
+            return []
+
+        # Check for new nodes added in the second_feature_map
+        new_added_nodes = []
+
+        # Checked for removed nodes in the first_feature_map
+        removed_nodes = []
+
+        # Checked for modified nodes
+        modified_nodes = []
+        for key, value in self._differences.get('values_changed').items():
+            modified_nodes.append(self.parsed_node_uuid(key))
+
+        # Merge unique nodeUuid of first_feature_map and second_feature_map
+        merged_node_uuids = set(self.first_feature_map.keys()).union(set(self.second_feature_map.keys()))
+        for node_uuid in merged_node_uuids:
+            if node_uuid not in self.first_feature_map:
+                new_added_nodes.append(node_uuid)
+            elif node_uuid not in self.second_feature_map:
+                removed_nodes.append(node_uuid)
+
+        return {
+            'new_added_nodes': new_added_nodes,
+            'removed_nodes': removed_nodes,
+            'existing_modified_nodes': modified_nodes
+        }
+
+    def get_difference_count(self):
+        """
+        :return: The total number of differences between the feature sets
+        """
+        return len(self._differences().keys())
+
+    def parsed_item_added(self):
+        item_added: Dict = self._differences.get('iterable_item_added')
+        if item_added:
+            return {}
+
+        for key, value in item_added.items():
+            node = self.parsed_node_uuid(key)
+            if node in self._changed_nodes['new_added_nodes']:
+                self._differences['iterable_item_added'][key]['details'] = f'Node: {node} was added'
+                continue
+
+            # if node in
+        return self.get_difference_count()
+
+    def parsed_node_uuid(self, key):
+        """
+        :param key: Key of data dictionary
+        :return: node uuid from the key
+        """
+        node = key.split("['")[1].split("']")[0]
+        return node
+
+
 class Document(object):
     """A Document is a collection of metadata and a set of content nodes."""
 
     PREVIOUS_VERSION: str = "1.0.0"
-    CURRENT_VERSION: str = "4.0.2"
+    CURRENT_VERSION: str = "6.0.0"
 
     def __str__(self):
         return f"kodexa://{self.uuid}"
@@ -1762,6 +1896,9 @@ class Document(object):
         self.classes: List[ContentClassification] = []
         """A list of the content classifications associated at the document level"""
 
+        self.tag_instances: List[TagInstance] = []
+        """A list of tag instances that contains a set of tag that has a set of nodes"""
+
         # Start persistence layer
         from kodexa.model import PersistenceManager
 
@@ -1770,6 +1907,36 @@ class Document(object):
                                                                                    delete_on_close=delete_on_close)
         self._persistence_layer.initialize()
 
+    def add_tag_instance(self, tag_to_apply, node_list: List[ContentNode]):
+        """
+            This will create a group of a tag with indexes
+        :param tag: name of the tag
+        :param node_indices: contains the list of index of a node
+        :return:
+        """
+        # For each node in the list create/update a feature
+        tag = Tag()
+        for node in node_list:
+            node.add_feature('tag', tag_to_apply, Tag)
+        # Tag Object
+        tag_instance = TagInstance(tag, node_list)
+        self.tag_instances.append(tag_instance)
+
+    def update_tag_instance(self, tag_uuid):
+        for tag_instance in self.tag_instances:
+            if tag_instance.tag.uuid == tag_uuid:
+                # Update attributes of a Tag
+                for node in tag_instance.nodes:
+                    node.get_tag(tag_instance.tag.value, tag_uuid=tag_instance.tag.uuid)
+
+    def get_tag_instance(self, tag):
+        """
+            Get the tag instance based on the tag itself
+        :param tag: name of the tag
+        :return: a list of tag instance
+        """
+        return [tag_instance for tag_instance in self.tag_instances if tag_instance.tag == tag]
+
     def get_persistence(self):
         return self._persistence_layer
 
@@ -2162,11 +2329,17 @@ class Document(object):
         """
         if isinstance(source, str):
             if isinstance(source, str):
-                document = Document(kddb_path=source)
-            if detached:
-                return Document.from_kddb(document.to_kddb())
 
-            return document
+                # If we are using the detached flag we will create a copy of the KDDB file
+                if detached:
+                    import tempfile
+                    from kodexa import KodexaPlatform
+                    fp = tempfile.NamedTemporaryFile(suffix='.kddb', delete=False, dir=KodexaPlatform.get_tempdir())
+                    fp.write(open(source, 'rb').read())
+                    fp.close()
+                    return Document(kddb_path=fp.name, delete_on_close=True)
+
+                return Document(kddb_path=source)
 
         # We will assume the input is of byte type
         import tempfile
@@ -2278,6 +2451,49 @@ class Document(object):
         """
         return self.labels
 
+    def get_feature_set(self) -> FeatureSet:
+        """
+        Build a feature set of all the tagged nodes
+
+        :return:
+        """
+        feature_set = FeatureSet()
+        feature_set.node_features = []
+        for tagged_node in self.get_all_tagged_nodes():
+            node_feature = {
+                'nodeUuid': str(tagged_node.uuid),
+                'features': []
+            }
+
+            feature_set.node_features.append(node_feature)
+
+            # TODO this needs to be cleaned up
+            for feature in tagged_node.get_features():
+                if feature.feature_type == 'tag':
+                    feature_dict = feature.to_dict()
+                    feature_dict['featureType'] = feature.feature_type
+                    feature_dict['name'] = feature.name
+                    node_feature['features'].append(feature_dict)
+
+        return feature_set
+
+    def get_all_tagged_nodes(self) -> List[ContentNode]:
+        """
+        Get all the tagged nodes in the document
+
+        :return:
+        """
+        return self._persistence_layer.get_all_tagged_nodes()
+
+
+class TagInstance:
+    def __init__(self, tag: Tag, nodes):
+        self.tag = tag
+        self.nodes = nodes
+
+    def add_node(self, nodes: List[ContentNode]):
+        self.nodes.extend(nodes)
+
 
 class ContentObjectReference:
     """ """
