@@ -15,7 +15,6 @@ import time
 from json import JSONDecodeError
 
 import requests
-import yaml
 from addict import Dict
 from appdirs import AppDirs
 
@@ -25,7 +24,7 @@ from kodexa.model import Document, ExtensionPack
 from kodexa.model.objects import AssistantDefinition, Action, Taxonomy, ModelRuntime, CredentialDefinition, \
     ExecutionEvent, \
     ContentObject, AssistantEvent, ContentEvent, ScheduledEvent, Project, Execution, ProjectTemplate, Membership, \
-    DataForm
+    DataForm, DocumentFamilyEvent, ChannelEvent, DataObjectEvent
 from kodexa.pipeline import PipelineContext, Pipeline, PipelineStatistics
 from kodexa.platform.client import DocumentStoreEndpoint, KodexaClient
 
@@ -34,7 +33,7 @@ logger = logging.getLogger()
 dirs = AppDirs("Kodexa", "Kodexa")
 
 
-def get_config():
+def get_config(profile=None):
     """Get the kodexa config object we use when you want to store your PAT locally
 
     :return: the config as a dict
@@ -47,9 +46,19 @@ def get_config():
     path = os.path.join(dirs.user_config_dir, '.kodexa.json')
     if os.path.exists(path):
         with open(path, 'r') as outfile:
-            return json.load(outfile)
+            kodexa_config = json.load(outfile)
+            if profile and profile not in kodexa_config:
+                kodexa_config[profile] = {'url': None, 'access_token': None, 'insecure': False}
+
+            if not profile and 'insecure' not in kodexa_config:
+                kodexa_config['insecure'] = False
+            elif profile and 'insecure' not in kodexa_config[profile]:
+                kodexa_config[profile]['insecure'] = False
+
+            return kodexa_config
     else:
-        return {'url': None, 'access_token': None}
+        return {'url': None, 'access_token': None, 'insecure': False} if not profile else {
+            profile: {'url': None, 'access_token': None, 'insecure': False}}
 
 
 def save_config(config_obj):
@@ -72,56 +81,6 @@ def save_config(config_obj):
         json.dump(config_obj, outfile)
 
 
-DEFAULT_COLUMNS = {
-    'extensionPacks': [
-        'ref',
-        'name',
-        'description',
-        'type',
-        'status'
-    ],
-    'projects': [
-        'id',
-        'organization.name',
-        'name',
-        'description'
-    ],
-    'assistants': [
-        'ref',
-        'name',
-        'description',
-        'template'
-    ],
-    'executions': [
-        'id',
-        'startDate',
-        'endDate',
-        'status',
-        'assistant.name',
-        'documentFamily.path'
-    ],
-    'memberships': [
-        'organization.slug',
-        'organization.name'
-    ],
-
-    'stores': [
-        'ref',
-        'name',
-        'description',
-        'storeType',
-        'storePurpose',
-        'template'
-    ],
-    'default': [
-        'ref',
-        'name',
-        'description',
-        'type',
-        'template'
-    ]
-}
-
 OBJECT_TYPES = {
     "extensionPacks": {
         "name": "extension pack",
@@ -244,7 +203,7 @@ class KodexaPlatform:
         return KodexaClient(KodexaPlatform.get_url(), KodexaPlatform.get_access_token())
 
     @staticmethod
-    def get_access_token() -> str:
+    def get_access_token(profile=None) -> str:
         """
         Returns the access token
 
@@ -253,12 +212,13 @@ class KodexaPlatform:
         Returns: The access token if it is defined in the user config store, or as an environment variable
 
         """
-        kodexa_config = get_config()
+        kodexa_config = get_config(profile)
         access_token = os.getenv('KODEXA_ACCESS_TOKEN')
-        return access_token if access_token is not None else kodexa_config['access_token']
+        return access_token if access_token is not None else \
+            kodexa_config[profile]['access_token'] if profile else kodexa_config['access_token']
 
     @staticmethod
-    def get_url() -> str:
+    def get_url(profile=None) -> str:
         """
         Returns the URL to use to access a Kodexa Platform
 
@@ -269,9 +229,9 @@ class KodexaPlatform:
         Returns: The URL if it is defined in the user config store, or as an environment variable
 
         """
-        kodexa_config = get_config()
+        kodexa_config = get_config(profile)
         env_url = os.getenv('KODEXA_URL', None)
-        return env_url if env_url is not None else kodexa_config['url']
+        return env_url if env_url is not None else kodexa_config[profile]['url'] if profile else kodexa_config['url']
 
     @staticmethod
     def set_access_token(access_token: str):
@@ -313,7 +273,8 @@ class KodexaPlatform:
         """
         response = requests.get(
             f"{KodexaPlatform.get_url()}/api/account/accessToken",
-            headers={"x-access-token": KodexaPlatform.get_access_token()})
+            headers={"x-access-token": KodexaPlatform.get_access_token()},
+            verify=not KodexaPlatform.get_insecure())
         if response.status_code == 200:
             return Dict(response.json())
 
@@ -336,15 +297,22 @@ class KodexaPlatform:
         return [org_slug, slug, version]
 
     @classmethod
-    def login(cls, kodexa_url, username, password):
+    def login(cls, kodexa_url, username, password, profile=None, insecure=False):
         from requests.auth import HTTPBasicAuth
         obj_response = requests.get(f"{kodexa_url}/api/account/me/token",
                                     auth=HTTPBasicAuth(username, password),
-                                    headers={"content-type": "application/json"})
+                                    headers={"content-type": "application/json"},
+                                    verify=not insecure)
         if obj_response.status_code == 200:
-            kodexa_config = get_config()
-            kodexa_config['url'] = kodexa_url
-            kodexa_config['access_token'] = obj_response.text
+            kodexa_config = get_config(profile)
+            if profile and profile in kodexa_config:
+                kodexa_config[profile]['url'] = kodexa_url
+                kodexa_config[profile]['access_token'] = obj_response.text
+                kodexa_config[profile]['insecure'] = insecure
+            else:
+                kodexa_config['url'] = kodexa_url
+                kodexa_config['access_token'] = obj_response.text
+                kodexa_config['insecure'] = insecure
             save_config(kodexa_config)
             print("Logged in")
         else:
@@ -355,7 +323,8 @@ class KodexaPlatform:
         """ """
         r = requests.get(f"{KodexaPlatform.get_url()}/api",
                          headers={"x-access-token": KodexaPlatform.get_access_token(),
-                                  "content-type": "application/json"})
+                                  "content-type": "application/json"},
+                         verify=not KodexaPlatform.get_insecure())
         if r.status_code == 401:
             raise Exception("Your access token was not authorized")
         if r.status_code == 200:
@@ -369,6 +338,13 @@ class KodexaPlatform:
         import tempfile
         return os.getenv('KODEXA_TMP', tempfile.gettempdir())
 
+    @classmethod
+    def get_insecure(cls, profile=None):
+        kodexa_config = get_config(profile)
+        insecure = os.getenv('KODEXA_URL_INSECURE', None)
+        return insecure if insecure is not None else kodexa_config[profile]['insecure'] if profile else kodexa_config[
+            'insecure']
+
 
 class RemoteSession:
     """A Session on the Kodexa platform for leveraging pipelines and services"""
@@ -389,7 +365,8 @@ class RemoteSession:
         """
         logger.debug(f"Downloading metadata for action {ref}")
         r = requests.get(f"{KodexaPlatform.get_url()}/api/actions/{ref.replace(':', '/')}",
-                         headers={"x-access-token": KodexaPlatform.get_access_token()})
+                         headers={"x-access-token": KodexaPlatform.get_access_token()},
+                         verify=not KodexaPlatform.get_insecure())
         if r.status_code == 401:
             raise Exception("Your access token was not authorized")
         if r.status_code == 200:
@@ -402,7 +379,8 @@ class RemoteSession:
         """ """
         logger.info(f"Creating session {self.slug} ({KodexaPlatform.get_url()})")
         r = requests.post(f"{KodexaPlatform.get_url()}/api/sessions", params={self.session_type: self.slug},
-                          headers={"x-access-token": KodexaPlatform.get_access_token()})
+                          headers={"x-access-token": KodexaPlatform.get_access_token()},
+                          verify=not KodexaPlatform.get_insecure())
 
         if r.status_code != 200:
             logger.warning("Unable to create session")
@@ -427,7 +405,8 @@ class RemoteSession:
         r = requests.post(f"{KodexaPlatform.get_url()}/api/sessions/{self.cloud_session.id}/execute",
                           params={self.session_type: self.slug, "documentVersion": document.version},
                           data=data,
-                          headers={"x-access-token": KodexaPlatform.get_access_token()}, files=files)
+                          headers={"x-access-token": KodexaPlatform.get_access_token()}, files=files,
+                          verify=not KodexaPlatform.get_insecure())
         try:
             if r.status_code == 200:
                 execution = Dict(json.loads(r.text))
@@ -445,7 +424,8 @@ class RemoteSession:
         while execution.status == "PENDING" or execution.status == "RUNNING":
             r = requests.get(
                 f"{KodexaPlatform.get_url()}/api/sessions/{self.cloud_session.id}/executions/{execution.id}",
-                headers={"x-access-token": KodexaPlatform.get_access_token()})
+                headers={"x-access-token": KodexaPlatform.get_access_token()},
+                verify=not KodexaPlatform.get_insecure())
             try:
                 execution = Dict(json.loads(r.text))
             except JSONDecodeError:
@@ -493,7 +473,8 @@ class RemoteSession:
             logger.info(f"Downloading output document [{execution.outputId}]")
             doc = requests.get(
                 f"{KodexaPlatform.get_url()}/api/sessions/{self.cloud_session.id}/executions/{execution.id}/objects/{execution.outputId}",
-                headers={"x-access-token": KodexaPlatform.get_access_token()})
+                headers={"x-access-token": KodexaPlatform.get_access_token()},
+                verify=KodexaPlatform.get_insecure())
             return Document.from_kddb(doc.content)
 
         logger.info("No output document")
@@ -701,26 +682,6 @@ class RemoteStep:
         }
 
 
-class ExtensionHelper:
-    """ """
-
-    @staticmethod
-    def load_metadata(path) -> dict:
-
-        if os.path.exists(os.path.join(path, 'dharma.json')):
-            dharma_metadata_file = open(os.path.join(path, 'dharma.json'))
-            dharma_metadata = json.loads(dharma_metadata_file.read())
-        elif os.path.exists(os.path.join(path, 'dharma.yml')):
-            dharma_metadata_file = open(os.path.join(path, 'dharma.yml'))
-            dharma_metadata = yaml.safe_load(dharma_metadata_file.read())
-        elif os.path.exists(os.path.join(path, 'kodexa.yml')):
-            dharma_metadata_file = open(os.path.join(path, 'kodexa.yml'))
-            dharma_metadata = yaml.safe_load(dharma_metadata_file.read())
-        else:
-            raise Exception("Unable to find a kodexa.yml file describing your extension")
-        return dharma_metadata
-
-
 class EventHelper:
 
     def __init__(self, event: ExecutionEvent):
@@ -734,6 +695,12 @@ class EventHelper:
             return ContentEvent(**event_dict)
         if event_dict['type'] == 'scheduled':
             return ScheduledEvent(**event_dict)
+        if event_dict['type'] == 'channel':
+            return ChannelEvent(**event_dict)
+        if event_dict['type'] == 'documentFamily':
+            return DocumentFamilyEvent(**event_dict)
+        if event_dict['type'] == 'dataObject':
+            return DataObjectEvent(**event_dict)
 
         raise f"Unknown event type {event_dict}"
 
@@ -743,7 +710,8 @@ class EventHelper:
             json=[
                 {'entry': message}
             ],
-            headers={'x-access-token': KodexaPlatform.get_access_token()}, timeout=300)
+            headers={'x-access-token': KodexaPlatform.get_access_token()}, timeout=300,
+            verify=not KodexaPlatform.get_insecure())
         if response.status_code != 200:
             print(f"Logging failed {response.status_code}", flush=True)
 
@@ -753,7 +721,8 @@ class EventHelper:
 
         co_response = requests.get(
             f"{KodexaPlatform.get_url()}/api/sessions/{self.event.session_id}/executions/{self.event.execution.id}/objects/{content_object_id}",
-            headers={'x-access-token': KodexaPlatform.get_access_token()}, timeout=300)
+            headers={'x-access-token': KodexaPlatform.get_access_token()}, timeout=300,
+            verify=not KodexaPlatform.get_insecure())
         if co_response.status_code != 200:
             logger.error(f"Response {co_response.status_code} {co_response.text}")
             raise Exception(f"Unable to find content object {content_object_id} in execution {self.event.execution.id}")
@@ -771,7 +740,8 @@ class EventHelper:
             f"{KodexaPlatform.get_url()}/api/sessions/{self.event.session_id}/executions/{self.event.execution.id}/objects",
             data=data,
             headers={'x-access-token': KodexaPlatform.get_access_token()},
-            files=files, timeout=300)
+            files=files, timeout=300,
+            verify=not KodexaPlatform.get_insecure())
 
         if co_response.status_code != 200:
             logger.info("Unable to post back object")
@@ -780,7 +750,7 @@ class EventHelper:
 
         logger.info("Object posted back")
 
-        return ContentObject.parse_obj(co_response.json())
+        return ContentObject.model_validate(co_response.json())
 
     def build_pipeline_context(self) -> PipelineContext:
         context = PipelineContext(context={}, content_provider=self, execution_id=self.event.execution.id)
