@@ -2,5 +2,5 @@
 Utilities to help support unit testing and test harnesses for Kodexa
 """
 
-from .test_components import *
+from .test_components import TestAction, TestAssistant
 from .test_utils import *
