@@ -1,6 +1,5 @@
-from kodexa.assistant import Assistant, AssistantContext, AssistantResponse
-from kodexa.assistant.assistant import AssistantPipeline
-from kodexa.model.model import BaseEvent
+from kodexa.assistant import Assistant, AssistantContext, AssistantResponse, AssistantPipeline
+from kodexa.model.objects import BaseEvent
 from kodexa.pipeline import Pipeline
 
 
@@ -10,10 +9,6 @@ class TestAction:
     def __init__(self, cheese: str = None):
         self.cheese = cheese
 
-    def get_name(self):
-        """ """
-        return "Hello"
-
 
 class TestAssistant(Assistant):
     """ """
@@ -35,4 +30,4 @@ class TestAssistant(Assistant):
         pipeline = Pipeline()
         pipeline.add_label('hello')
 
-        return AssistantResponse([AssistantPipeline(pipeline, write_back_to_store=True)])
+        return AssistantResponse(pipelines=[AssistantPipeline(pipeline=pipeline, write_back_to_store=True)])
